;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                ;;
;;  Mode pour le langage vaλisp   ;;
;;                                ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun load-upinfo-repl-valisp ()
  "Mode pour travailler avec le langage vaλisp"
  (interactive)
  (emacs-lisp-mode)
  (hs-show-all)
  (load "upinfo-repl-valisp")
  (valisp-repl-add-keybinding))

(defun valisp-repl-add-keybinding()
  (interactive)
  (local-set-key (kbd "C-x C-a") 'valisp-send-previous-sexpr)
  (local-set-key (kbd "C-x a") 'valisp-send-previous-sexpr-repeat)
  (local-set-key (kbd "<f5>") 'valisp-send-defun)
  (local-set-key (kbd "<f8>") 'valisp-change-commande)
  (local-set-key (kbd "<f9>") 'valisp-run)
  (local-set-key (kbd "C-c RET") 'hs-toggle-hiding))

(defvar *valisp-commande-historique* nil)





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                            ;;
;;  Démarrage & redémarrage   ;;
;;                            ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun valisp-init (&optional ask)
  (interactive)
  (let ((commande (if ask (read-shell-command
                           "votre programme> "
                           (or (car *valisp-commande-historique*) "./valisp ")
                           '*valisp-commande-historique*)
                    (car *valisp-commande-historique*))))
    (let* ((liste (split-string-and-unquote commande))
           (cmd (car liste))
           (arg (cdr liste)))
      (apply #'make-comint (cl-list* "valisp" cmd nil arg))))
  (display-buffer "*valisp*"))

(defun valisp-run (&optional ask)
  (interactive)
  (when (get-buffer "*valisp*")
    (with-current-buffer "*valisp*"
      (when (comint-check-proc "*valisp*")
        (comint-quit-subjob)
        (sleep-for 0.1))
      (erase-buffer)))
  (valisp-init ask))

(defun valisp-change-commande ()
  (interactive)
  (valisp-run t))

(defun valisp-run-first-time ()
  (interactive)
  (unless (and (get-buffer "*valisp*")
               (comint-check-proc "*valisp*"))
    (valisp-run t))
  (display-buffer "*valisp*"))





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                    ;;
;;  Envoie des commandes vers valisp  ;;
;;                                    ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun repeat-command (command)
  "Repeat COMMAND."
  (require 'repeat)
  (let ((repeat-previous-repeated-command  command)
        (repeat-message-function           #'ignore)
        (last-repeatable-command           'repeat))
    (repeat nil)))


(defun valisp-send-previous-sexpr ()
  (interactive)
  (let ((sexpr (thing-at-point 'sexp t)))
    (valisp-run-first-time)
    (with-current-buffer "*valisp*"
      (goto-char (point-max))
      (insert sexpr)
      (comint-send-input))))


(defun valisp-send-previous-sexpr-repeat()
  (interactive)
  (repeat-command 'valisp-send-previous-sexpr)
  (forward-sexp))



(provide 'upinfo-repl-valisp)
